@@ -830,7 +830,7 @@ let query_package () =
       "-qo", Arg.Set qo,
           "              do not print regular output";
     ]
-    (fun p -> packages := !packages @ [p])
+    (fun p -> packages := !packages @ Fl_split.in_words p)
 "usage: ocamlfind query [ -predicates <p>  | -format <f> |
                          -long-format     | -i-format   |
                          -l-format        | -a-format   |
@@ -839,6 +839,7 @@ let query_package () =
                          -separator <s>   |
                          -descendants     | -recursive  ] package ...";
 
+  ignore(config_file());  (* ensure findlib is initialized *)
   try
     let predicates1 =
       if !pp then
@@ -930,6 +931,9 @@ let ocamlc which () =
       | _       -> `None
   in
   let threads = ref `None in
+  let support_threads() =
+    if threads_default = `None then
+      failwith "threading is not supported on this platform" in
 
   let add_switch name =
     Arg.Unit (fun () ->
@@ -1026,10 +1030,10 @@ let ocamlc which () =
 	      Arg.String (fun s -> pp_specified := true; add_spec_fn "-pp" s);
 	      
 	      "-thread", 
-	      Arg.Unit (fun _ -> threads := threads_default);
+              Arg.Unit (fun _ -> support_threads(); threads := threads_default);
             
 	      "-vmthread", 
-	      Arg.Unit (fun _ -> threads := `VM_threads);
+              Arg.Unit (fun _ -> support_threads(); threads := `VM_threads);
               
 	      "-", 
 	      Arg.String (fun s -> pass_files := !pass_files @  [ Pass s ]);
@@ -1152,6 +1156,18 @@ let ocamlc which () =
   (* Add the pkg_<name> predicates: *)
   predicates := List.map (fun pkg -> "pkg_" ^ pkg) eff_packages @ !predicates;
 
+  (* Check on [warning] directives: *)
+  List.iter
+    (fun pkg ->
+       try
+         let warning = package_property !predicates pkg "warning" in
+         prerr_endline("ocamlfind: [WARNING] Package `" ^ pkg ^
+                         "': " ^ warning)
+       with
+	   Not_found -> ()
+    )
+    eff_packages;
+
   (* Check on [error] directives: *)
   List.iter
     (fun pkg ->
@@ -1853,6 +1869,12 @@ let create_owner_file pkg file =
       exc -> close_out f; raise exc
 ;;
 
+let trim_cr s =
+  let len = String.length s in
+  if len > 0 && String.get s (len-1) = '\r' then
+    String.sub s 0 (len-1)
+  else
+    s
 
 let find_owned_files pkg dir =
   let files = Array.to_list(Sys.readdir dir) in
@@ -1870,7 +1892,7 @@ let find_owned_files pkg dir =
            let f =
              Unix.in_channel_of_descr fd in
            try
-             let line = input_line f in
+             let line = trim_cr (input_line f) in
              let is_my_file = (line = pkg) in
              close_in f;
              is_my_file
@@ -1997,6 +2019,17 @@ let meta_pkg meta_name =
     close_in f;
     failwith ("Cannot parse '" ^ meta_name ^ "': " ^ s)
 
+let char_lowercase_ascii c =
+  (* Char.lowercase_ascii and String.lowercase_ascii first available in
+     OCaml-4.03, but we want to support earlier versions too
+   *)
+  if (c >= 'A' && c <= 'Z')
+  then Char.unsafe_chr(Char.code c + 32)
+  else c
+
+let string_lowercase_ascii =
+  String.map char_lowercase_ascii
+
 
 let install_package () =
   let destdir = ref (default_location()) in
@@ -2180,17 +2213,15 @@ let install_package () =
     (* FIXME: We have to be careful with case-insensitive filesystems. 
        Currently, we only check for Win32, but also OS X may have ci 
        filesystems. So some better check would be nice.
-       Furthermore, String.lowercase assumes that the encoding of file names is
-       ISO-8859-1. This is probably plainly wrong.
      *)
     let lines = read_ldconf !ldconf in
     let dlldir_norm = Fl_split.norm_dir dlldir in
-    let dlldir_norm_lc = String.lowercase dlldir_norm in
+    let dlldir_norm_lc = string_lowercase_ascii dlldir_norm in
     let ci_filesys = (Sys.os_type = "Win32") in
     let check_dir d =
       let d' = Fl_split.norm_dir d in
       (d' = dlldir_norm) || 
-	(ci_filesys && String.lowercase d' = dlldir_norm_lc) in
+        (ci_filesys && string_lowercase_ascii d' = dlldir_norm_lc) in
     if not (List.exists check_dir lines) then
       prerr_endline("ocamlfind: [WARNING] You have installed DLLs but the directory " ^ dlldir_norm ^ " is not mentioned in ld.conf");
   end;
@@ -2436,7 +2467,7 @@ let print_configuration() =
 	  (Findlib.ocaml_ldconf());
 	flush stdout
     | Some "conf" ->
-	print_endline Findlib_config.config_file
+	print_endline (Findlib.config_file())
     | Some "path" ->
 	List.iter print_endline (Findlib.search_path())
     | Some "destdir" ->
